@@ -14,10 +14,6 @@
 #include <Sonnet/BackgroundChecker>
 #include <Sonnet/Settings>
 
-#include "models/actionsmodel.h"
-#include "models/roomlistmodel.h"
-#include "neochatroom.h"
-
 class SyntaxHighlighter : public QSyntaxHighlighter
 {
 public:
@@ -138,12 +134,13 @@ int ChatDocumentHandler::completionStartIndex() const
         return 0;
     }
 
-#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
+#if !defined(Q_OS_ANDROID) && QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
     const long long cursor = cursorPosition();
 #else
     const auto cursor = cursorPosition();
 #endif
     const auto &text = getText();
+
     auto start = std::min(cursor, text.size()) - 1;
     while (start > -1) {
         if (text.at(start) == QLatin1Char(' ')) {
@@ -221,8 +218,8 @@ void ChatDocumentHandler::setRoom(NeoChatRoom *room)
 void ChatDocumentHandler::complete(int index)
 {
     if (m_completionModel->autoCompletionType() == CompletionModel::User) {
-        auto name = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::Text).toString();
-        auto id = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::Subtitle).toString();
+        auto name = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::DisplayNameRole).toString();
+        auto id = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::SubtitleRole).toString();
         auto text = getText();
         auto at = text.lastIndexOf(QLatin1Char('@'), cursorPosition() - 1);
         QTextCursor cursor(document()->textDocument());
@@ -235,7 +232,7 @@ void ChatDocumentHandler::complete(int index)
         pushMention({cursor, name, 0, 0, id});
         m_highlighter->rehighlight();
     } else if (m_completionModel->autoCompletionType() == CompletionModel::Command) {
-        auto command = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::ReplacedText).toString();
+        auto command = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::ReplacedTextRole).toString();
         auto text = getText();
         auto at = text.lastIndexOf(QLatin1Char('/'));
         QTextCursor cursor(document()->textDocument());
@@ -243,7 +240,7 @@ void ChatDocumentHandler::complete(int index)
         cursor.setPosition(cursorPosition(), QTextCursor::KeepAnchor);
         cursor.insertText(QStringLiteral("/%1 ").arg(command));
     } else if (m_completionModel->autoCompletionType() == CompletionModel::Room) {
-        auto alias = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::Subtitle).toString();
+        auto alias = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::SubtitleRole).toString();
         auto text = getText();
         auto at = text.lastIndexOf(QLatin1Char('#'), cursorPosition() - 1);
         QTextCursor cursor(document()->textDocument());
@@ -256,7 +253,7 @@ void ChatDocumentHandler::complete(int index)
         pushMention({cursor, alias, 0, 0, alias});
         m_highlighter->rehighlight();
     } else if (m_completionModel->autoCompletionType() == CompletionModel::Emoji) {
-        auto shortcode = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::ReplacedText).toString();
+        auto shortcode = m_completionModel->data(m_completionModel->index(index, 0), CompletionModel::ReplacedTextRole).toString();
         auto text = getText();
         auto at = text.lastIndexOf(QLatin1Char(':'));
         QTextCursor cursor(document()->textDocument());
@@ -324,3 +321,37 @@ void ChatDocumentHandler::pushMention(const Mention mention) const
         m_room->mentions()->push_back(mention);
     }
 }
+
+QColor ChatDocumentHandler::mentionColor() const
+{
+    return m_mentionColor;
+}
+
+void ChatDocumentHandler::setMentionColor(const QColor &color)
+{
+    if (m_mentionColor == color) {
+        return;
+    }
+    m_mentionColor = color;
+    m_highlighter->mentionFormat.setForeground(m_mentionColor);
+    m_highlighter->rehighlight();
+    Q_EMIT mentionColorChanged();
+}
+
+QColor ChatDocumentHandler::errorColor() const
+{
+    return m_errorColor;
+}
+
+void ChatDocumentHandler::setErrorColor(const QColor &color)
+{
+    if (m_errorColor == color) {
+        return;
+    }
+    m_errorColor = color;
+    m_highlighter->errorFormat.setForeground(m_errorColor);
+    m_highlighter->rehighlight();
+    Q_EMIT errorColorChanged();
+}
+
+#include "moc_chatdocumenthandler.cpp"
