@@ -31,23 +31,15 @@
 
 #include <signal.h>
 
-#ifdef QUOTIENT_07
-#include "accountregistry.h"
-#else
-#include "neochataccountregistry.h"
-#endif
-
-#include <connection.h>
-#include <csapi/content-repo.h>
-#include <csapi/logout.h>
-#include <csapi/profile.h>
-#include <jobs/downloadfilejob.h>
-#include <qt_connection_util.h>
-
-#ifdef QUOTIENT_07
-#include <csapi/notifications.h>
-#include <eventstats.h>
-#endif
+#include <Quotient/accountregistry.h>
+#include <Quotient/connection.h>
+#include <Quotient/csapi/content-repo.h>
+#include <Quotient/csapi/logout.h>
+#include <Quotient/csapi/profile.h>
+#include <Quotient/jobs/downloadfilejob.h>
+#include <Quotient/qt_connection_util.h>
+#include <Quotient/csapi/notifications.h>
+#include <Quotient/eventstats.h>
 
 #include "neochatconfig.h"
 #include "neochatroom.h"
@@ -73,24 +65,8 @@ Controller::Controller(QObject *parent)
     setApplicationProxy();
 
 #ifndef Q_OS_ANDROID
-    if (NeoChatConfig::self()->systemTray()) {
-        m_trayIcon = new TrayIcon(this);
-        m_trayIcon->show();
-        connect(m_trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
-        QGuiApplication::setQuitOnLastWindowClosed(false);
-    }
-    connect(NeoChatConfig::self(), &NeoChatConfig::SystemTrayChanged, this, [this]() {
-        if (NeoChatConfig::self()->systemTray()) {
-            m_trayIcon = new TrayIcon(this);
-            m_trayIcon->show();
-            connect(m_trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
-        } else {
-            disconnect(m_trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
-            delete m_trayIcon;
-            m_trayIcon = nullptr;
-        }
-        QGuiApplication::setQuitOnLastWindowClosed(!NeoChatConfig::self()->systemTray());
-    });
+    setQuitOnLastWindowClosed();
+    connect(NeoChatConfig::self(), &NeoChatConfig::SystemTrayChanged, this, &Controller::setQuitOnLastWindowClosed);
 #endif
 
     QTimer::singleShot(0, this, [this] {
@@ -125,97 +101,23 @@ Controller::Controller(QObject *parent)
     }
 #endif
 
-    connect(&AccountRegistry::instance(), &AccountRegistry::accountCountChanged, this, &Controller::activeConnectionIndexChanged);
+    connect(&m_accountRegistry, &AccountRegistry::accountCountChanged, this, &Controller::activeConnectionIndexChanged);
 
-#ifdef QUOTIENT_07
     static int oldAccountCount = 0;
-    connect(&AccountRegistry::instance(), &AccountRegistry::accountCountChanged, this, [=]() {
-        if (AccountRegistry::instance().size() > oldAccountCount) {
-            auto connection = AccountRegistry::instance().accounts()[AccountRegistry::instance().size() - 1];
-            connect(connection, &Connection::syncDone, this, [=]() {
-                handleNotifications(connection);
+    connect(&m_accountRegistry, &AccountRegistry::accountCountChanged, this, [this]() {
+        if (m_accountRegistry.size() > oldAccountCount) {
+            auto connection = m_accountRegistry.accounts()[m_accountRegistry.size() - 1];
+            connect(connection, &Connection::syncDone, this, [connection]() {
+                NotificationsManager::instance().handleNotifications(connection);
             });
         }
-        oldAccountCount = AccountRegistry::instance().size();
+        oldAccountCount = m_accountRegistry.size();
     });
-#endif
-}
 
-#ifdef QUOTIENT_07
-void Controller::handleNotifications(QPointer<Quotient::Connection> connection)
-{
-    static QStringList initial;
-    static QStringList oldNotifications;
-    auto job = connection->callApi<GetNotificationsJob>();
-
-    connect(job, &BaseJob::success, this, [job, connection]() {
-        const auto notifications = job->jsonData()["notifications"].toArray();
-        if (!initial.contains(connection->user()->id())) {
-            initial.append(connection->user()->id());
-            for (const auto &n : notifications) {
-                oldNotifications += n.toObject()["event"].toObject()["event_id"].toString();
-            }
-            return;
-        }
-        for (const auto &n : notifications) {
-            const auto notification = n.toObject();
-            if (notification["read"].toBool()) {
-                oldNotifications.removeOne(notification["event"].toObject()["event_id"].toString());
-                continue;
-            }
-            if (oldNotifications.contains(notification["event"].toObject()["event_id"].toString())) {
-                continue;
-            }
-            oldNotifications += notification["event"].toObject()["event_id"].toString();
-            auto room = connection->room(notification["room_id"].toString());
-
-            // If room exists, room is NOT active OR the application is NOT active, show notification
-            if (room
-                && !(RoomManager::instance().currentRoom() && room->id() == RoomManager::instance().currentRoom()->id()
-                     && QGuiApplication::applicationState() == Qt::ApplicationActive)) {
-                // The room might have been deleted (for example rejected invitation).
-                auto sender = room->user(notification["event"].toObject()["sender"].toString());
-
-                QString body;
-
-                if (notification["event"].toObject()["type"].toString() == "org.matrix.msc3381.poll.start") {
-                    body = notification["event"]
-                               .toObject()["content"]
-                               .toObject()["org.matrix.msc3381.poll.start"]
-                               .toObject()["question"]
-                               .toObject()["body"]
-                               .toString();
-                } else {
-                    body = notification["event"].toObject()["content"].toObject()["body"].toString();
-                }
-
-                if (notification["event"]["type"] == "m.room.encrypted") {
-#ifdef Quotient_E2EE_ENABLED
-                    auto decrypted = connection->decryptNotification(notification);
-                    body = decrypted["content"].toObject()["body"].toString();
-#endif
-                    if (body.isEmpty()) {
-                        body = i18n("Encrypted Message");
-                    }
-                }
-
-                QImage avatar_image;
-                if (!sender->avatarUrl(room).isEmpty()) {
-                    avatar_image = sender->avatar(128, room);
-                } else {
-                    avatar_image = room->avatar(128);
-                }
-                NotificationsManager::instance().postNotification(dynamic_cast<NeoChatRoom *>(room),
-                                                                  sender->displayname(room),
-                                                                  body,
-                                                                  avatar_image,
-                                                                  notification["event"].toObject()["event_id"].toString(),
-                                                                  true);
-            }
-        }
+    QTimer::singleShot(0, this, [this] {
+        m_pushRuleModel = new PushRuleModel;
     });
 }
-#endif
 
 Controller &Controller::instance()
 {
@@ -228,38 +130,6 @@ void Controller::showWindow()
     WindowController::instance().showAndRaiseWindow(QString());
 }
 
-void Controller::loginWithAccessToken(const QString &serverAddr, const QString &user, const QString &token, const QString &deviceName)
-{
-    if (user.isEmpty() || token.isEmpty()) {
-        return;
-    }
-
-    QUrl serverUrl(serverAddr);
-
-    auto conn = new Connection();
-    if (serverUrl.isValid()) {
-        conn->setHomeserver(serverUrl);
-    }
-
-    connect(conn, &Connection::connected, this, [this, conn, deviceName] {
-        AccountSettings account(conn->userId());
-        account.setKeepLoggedIn(true);
-        account.setHomeserver(conn->homeserver());
-        account.setDeviceId(conn->deviceId());
-        account.setDeviceName(deviceName);
-        if (!saveAccessTokenToKeyChain(account, conn->accessToken())) {
-            qWarning() << "Couldn't save access token";
-        }
-        account.sync();
-        addConnection(conn);
-        setActiveConnection(conn);
-    });
-    connect(conn, &Connection::networkError, this, [this](QString error, const QString &, int, int) {
-        Q_EMIT errorOccured(i18n("Network Error: %1", error));
-    });
-    conn->assumeIdentity(user, token, deviceName);
-}
-
 void Controller::logout(Connection *conn, bool serverSideLogout)
 {
     if (!conn) {
@@ -277,10 +147,10 @@ void Controller::logout(Connection *conn, bool serverSideLogout)
     job.start();
     loop.exec();
 
-    if (AccountRegistry::instance().count() > 1) {
+    if (m_accountRegistry.count() > 1) {
         // Only set the connection if the the account being logged out is currently active
         if (conn == activeConnection()) {
-            setActiveConnection(AccountRegistry::instance().accounts()[0]);
+            setActiveConnection(m_accountRegistry.accounts()[0]);
         }
     } else {
         setActiveConnection(nullptr);
@@ -295,15 +165,11 @@ void Controller::addConnection(Connection *c)
 {
     Q_ASSERT_X(c, __FUNCTION__, "Attempt to add a null connection");
 
-#ifndef QUOTIENT_07
-    AccountRegistry::instance().add(c);
-#endif
+    m_accountRegistry.add(c);
 
     c->setLazyLoading(true);
 
     connect(c, &Connection::syncDone, this, [this, c] {
-        setBusy(false);
-
         Q_EMIT syncDone();
 
         c->sync(30000);
@@ -314,13 +180,11 @@ void Controller::addConnection(Connection *c)
     });
 
     connect(c, &Connection::requestFailed, this, [this](BaseJob *job) {
-        if (job->error() == BaseJob::UserConsentRequiredError) {
+        if (job->error() == BaseJob::UserConsentRequired) {
             Q_EMIT userConsentRequired(job->errorUrl());
         }
     });
 
-    setBusy(true);
-
     c->sync();
 
     Q_EMIT connectionAdded(c);
@@ -331,15 +195,8 @@ void Controller::dropConnection(Connection *c)
 {
     Q_ASSERT_X(c, __FUNCTION__, "Attempt to drop a null connection");
 
-#ifndef QUOTIENT_07
-    AccountRegistry::instance().drop(c);
-#endif
-
     Q_EMIT connectionDropped(c);
     Q_EMIT accountCountChanged();
-#ifndef QUOTIENT_07
-    c->deleteLater();
-#endif
 }
 
 void Controller::invokeLogin()
@@ -390,7 +247,7 @@ void Controller::invokeLogin()
                 connect(connection, &Connection::networkError, this, [this](const QString &error, const QString &, int, int) {
                     Q_EMIT errorOccured(i18n("Network Error: %1", error));
                 });
-                connection->assumeIdentity(account.userId(), accessToken, account.deviceId());
+                connection->assumeIdentity(account.userId(), accessToken);
             });
         }
     }
@@ -406,7 +263,7 @@ QKeychain::ReadPasswordJob *Controller::loadAccessTokenFromKeyChain(const Accoun
     job->setKey(account.userId());
 
     // Handling of errors
-    connect(job, &QKeychain::Job::finished, this, [this, &account, job]() {
+    connect(job, &QKeychain::Job::finished, this, [this, job]() {
         if (job->error() == QKeychain::Error::NoError) {
             return;
         }
@@ -456,15 +313,9 @@ bool Controller::saveAccessTokenToKeyChain(const AccountSettings &account, const
 void Controller::changeAvatar(Connection *conn, const QUrl &localFile)
 {
     auto job = conn->uploadFile(localFile.toLocalFile());
-#ifdef QUOTIENT_07
-    if (isJobPending(job)) {
-#else
-    if (isJobRunning(job)) {
-#endif
-        connect(job, &BaseJob::success, this, [conn, job] {
-            conn->callApi<SetAvatarUrlJob>(conn->userId(), job->contentUri());
-        });
-    }
+    connect(job, &BaseJob::success, this, [conn, job] {
+        conn->callApi<SetAvatarUrlJob>(conn->userId(), job->contentUri());
+    });
 }
 
 void Controller::markAllMessagesAsRead(Connection *conn)
@@ -518,11 +369,7 @@ bool Controller::setAvatar(Connection *connection, const QUrl &avatarSource)
     User *localUser = connection->user();
     QString decoded = avatarSource.path();
     if (decoded.isEmpty()) {
-#ifdef QUOTIENT_07
         connection->callApi<SetAvatarUrlJob>(localUser->id(), avatarSource);
-#else
-        connection->callApi<SetAvatarUrlJob>(localUser->id(), QString());
-#endif
         return true;
     }
     if (QImageReader(decoded).read().isNull()) {
@@ -533,11 +380,7 @@ bool Controller::setAvatar(Connection *connection, const QUrl &avatarSource)
 }
 
 NeochatChangePasswordJob::NeochatChangePasswordJob(const QString &newPassword, bool logoutDevices, const Omittable<QJsonObject> &auth)
-#ifdef QUOTIENT_07
     : BaseJob(HttpVerb::Post, QStringLiteral("ChangePasswordJob"), "/_matrix/client/r0/account/password")
-#else
-    : BaseJob(HttpVerb::Post, QStringLiteral("ChangePasswordJob"), QStringLiteral("/_matrix/client/r0/account/password"))
-#endif
 {
     QJsonObject _data;
     addParam<>(_data, QStringLiteral("new_password"), newPassword);
@@ -548,34 +391,27 @@ NeochatChangePasswordJob::NeochatChangePasswordJob(const QString &newPassword, b
 
 int Controller::accountCount() const
 {
-    return AccountRegistry::instance().count();
+    return m_accountRegistry.count();
 }
 
-bool Controller::quitOnLastWindowClosed()
+void Controller::setQuitOnLastWindowClosed()
 {
-    return QGuiApplication::quitOnLastWindowClosed();
-}
-
-void Controller::setQuitOnLastWindowClosed(bool value)
-{
-    if (quitOnLastWindowClosed() != value) {
-        QGuiApplication::setQuitOnLastWindowClosed(value);
-        Q_EMIT quitOnLastWindowClosedChanged();
+#ifndef Q_OS_ANDROID
+    if (NeoChatConfig::self()->systemTray()) {
+        m_trayIcon = new TrayIcon(this);
+        m_trayIcon->show();
+        connect(m_trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
+    } else {
+        if (m_trayIcon) {
+            disconnect(m_trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
+            delete m_trayIcon;
+            m_trayIcon = nullptr;
+        }
     }
-}
-
-bool Controller::busy() const
-{
-    return m_busy;
-}
-
-void Controller::setBusy(bool busy)
-{
-    if (m_busy == busy) {
-        return;
-    }
-    m_busy = busy;
-    Q_EMIT busyChanged();
+    QGuiApplication::setQuitOnLastWindowClosed(!NeoChatConfig::self()->systemTray());
+#else
+    return;
+#endif
 }
 
 Connection *Controller::activeConnection() const
@@ -593,6 +429,7 @@ void Controller::setActiveConnection(Connection *connection)
     }
     if (m_connection != nullptr) {
         disconnect(m_connection, &Connection::syncError, this, nullptr);
+        disconnect(m_connection, &Connection::accountDataChanged, this, nullptr);
     }
     m_connection = connection;
     if (connection != nullptr) {
@@ -611,17 +448,28 @@ void Controller::setActiveConnection(Connection *connection)
             m_isOnline = true;
             Q_EMIT isOnlineChanged(true);
         });
-        connect(connection, &Connection::requestFailed, this, [=](BaseJob *job) {
+        connect(connection, &Connection::requestFailed, this, [](BaseJob *job) {
             if (dynamic_cast<DownloadFileJob *>(job) && job->jsonData()["errcode"].toString() == "M_TOO_LARGE"_ls) {
                 RoomManager::instance().warning(i18n("File too large to download."), i18n("Contact your matrix server administrator for support."));
             }
         });
+        connect(connection, &Connection::accountDataChanged, this, [this](const QString &type) {
+            if (type == QLatin1String("org.kde.neochat.account_label")) {
+                Q_EMIT activeAccountLabelChanged();
+            }
+        });
     } else {
         NeoChatConfig::self()->setActiveConnection(QString());
     }
     NeoChatConfig::self()->save();
     Q_EMIT activeConnectionChanged();
     Q_EMIT activeConnectionIndexChanged();
+    Q_EMIT activeAccountLabelChanged();
+}
+
+PushRuleModel *Controller::pushRuleModel() const
+{
+    return m_pushRuleModel;
 }
 
 void Controller::saveWindowGeometry()
@@ -630,11 +478,7 @@ void Controller::saveWindowGeometry()
 }
 
 NeochatDeleteDeviceJob::NeochatDeleteDeviceJob(const QString &deviceId, const Omittable<QJsonObject> &auth)
-#ifdef QUOTIENT_07
     : Quotient::BaseJob(HttpVerb::Delete, QStringLiteral("DeleteDeviceJob"), QStringLiteral("/_matrix/client/r0/devices/%1").arg(deviceId).toLatin1())
-#else
-    : Quotient::BaseJob(HttpVerb::Delete, QStringLiteral("DeleteDeviceJob"), QStringLiteral("/_matrix/client/r0/devices/%1").arg(deviceId))
-#endif
 {
     QJsonObject _data;
     addParam<IfNotEmpty>(_data, QStringLiteral("auth"), auth);
@@ -645,16 +489,30 @@ void Controller::createRoom(const QString &name, const QString &topic)
 {
     auto createRoomJob = m_connection->createRoom(Connection::PublishRoom, "", name, topic, QStringList());
     connect(createRoomJob, &CreateRoomJob::failure, this, [this, createRoomJob] {
-        Q_EMIT errorOccured(i18n("Room creation failed: \"%1\"", createRoomJob->errorString()));
+        Q_EMIT errorOccured(i18n("Room creation failed: %1", createRoomJob->errorString()));
     });
-    connectSingleShot(
-        this,
-        &Controller::roomAdded,
-        this,
-        [this](NeoChatRoom *room) {
-            RoomManager::instance().enterRoom(room);
-        },
-        Qt::QueuedConnection);
+    connectSingleShot(this, &Controller::roomAdded, &RoomManager::instance(), &RoomManager::enterRoom, Qt::QueuedConnection);
+}
+
+void Controller::createSpace(const QString &name, const QString &topic)
+{
+    auto createRoomJob = m_connection->createRoom(Connection::UnpublishRoom,
+                                                  {},
+                                                  name,
+                                                  topic,
+                                                  QStringList(),
+                                                  {},
+                                                  {},
+                                                  false,
+                                                  {},
+                                                  {},
+                                                  QJsonObject{
+                                                      {"type"_ls, "m.space"_ls},
+                                                  });
+    connect(createRoomJob, &CreateRoomJob::failure, this, [this, createRoomJob] {
+        Q_EMIT errorOccured(i18n("Space creation failed: %1", createRoomJob->errorString()));
+    });
+    connectSingleShot(this, &Controller::roomAdded, &RoomManager::instance(), &RoomManager::enterRoom, Qt::QueuedConnection);
 }
 
 bool Controller::isOnline() const
@@ -724,18 +582,9 @@ bool Controller::hasWindowSystem() const
 #endif
 }
 
-QString Controller::plainText(QQuickTextDocument *document) const
-{
-    return document->textDocument()->toPlainText();
-}
-
 bool Controller::encryptionSupported() const
 {
-#ifdef QUOTIENT_07
     return Quotient::encryptionSupported();
-#else
-    return false;
-#endif
 }
 
 void Controller::forceRefreshTextDocument(QQuickTextDocument *textDocument, QQuickItem *item)
@@ -776,29 +625,16 @@ void Controller::setApplicationProxy()
 
 int Controller::activeConnectionIndex() const
 {
-#ifdef QUOTIENT_07
-    auto result = std::find_if(Accounts.accounts().begin(), Accounts.accounts().end(), [this](const auto &it) {
+    auto result = std::find_if(m_accountRegistry.accounts().begin(), m_accountRegistry.accounts().end(), [this](const auto &it) {
         return it == m_connection;
     });
-    return result - Accounts.accounts().begin();
-#else
-    for (int i = 0; i < AccountRegistry::instance().rowCount(); i++) {
-        if (AccountRegistry::instance().data(AccountRegistry::instance().index(i, 0), AccountRegistry::UserIdRole).toString() == m_connection->userId()) {
-            return i;
-        }
-    }
-    return 0;
-#endif
+    return result - m_accountRegistry.accounts().begin();
 }
 
 int Controller::quotientMinorVersion() const
 {
-// TODO libQuotient 0.7: Replace with version function from libQuotient
-#ifdef QUOTIENT_07
+    // TODO libQuotient 0.7: Replace with version function from libQuotient
     return 7;
-#else
-    return 6;
-#endif
 }
 
 bool Controller::isFlatpak() const
@@ -809,3 +645,45 @@ bool Controller::isFlatpak() const
     return false;
 #endif
 }
+
+void Controller::setActiveAccountLabel(const QString &label)
+{
+    if (!m_connection) {
+        return;
+    }
+    QJsonObject json{
+        {"account_label", label},
+    };
+    m_connection->setAccountData("org.kde.neochat.account_label", json);
+}
+
+QString Controller::activeAccountLabel() const
+{
+    if (!m_connection) {
+        return {};
+    }
+    return m_connection->accountDataJson("org.kde.neochat.account_label")["account_label"].toString();
+}
+
+QVariantList Controller::getSupportedRoomVersions(Quotient::Connection *connection)
+{
+    auto roomVersions = connection->availableRoomVersions();
+
+    QVariantList supportedRoomVersions;
+    for (const Quotient::Connection::SupportedRoomVersion &v : roomVersions) {
+        QVariantMap roomVersionMap;
+        roomVersionMap.insert("id", v.id);
+        roomVersionMap.insert("status", v.status);
+        roomVersionMap.insert("isStable", v.isStable());
+        supportedRoomVersions.append(roomVersionMap);
+    }
+
+    return supportedRoomVersions;
+}
+
+AccountRegistry &Controller::accounts()
+{
+    return m_accountRegistry;
+}
+
+#include "moc_controller.cpp"
