@@ -13,7 +13,7 @@
 #include <KLocalizedString>
 
 #include "controller.h"
-#include <connection.h>
+#include <Quotient/connection.h>
 
 using namespace Quotient;
 
@@ -32,11 +32,17 @@ ThumbnailResponse::ThumbnailResponse(QString id, QSize size)
         requestedSize.setWidth(100);
     }
     if (mediaId.count('/') != 1) {
-        errorStr = i18n("Media id '%1' doesn't follow server/mediaId pattern", mediaId);
-        Q_EMIT finished();
-        return;
+        if (mediaId.startsWith(QLatin1Char('/'))) {
+            mediaId = mediaId.mid(1);
+        } else {
+            errorStr = i18n("Media id '%1' doesn't follow server/mediaId pattern", mediaId);
+            Q_EMIT finished();
+            return;
+        }
     }
 
+    mediaId = mediaId.split(QLatin1Char('?'))[0];
+
     QImage cachedImage;
     if (cachedImage.load(localFile)) {
         image = cachedImage;
@@ -98,18 +104,6 @@ void ThumbnailResponse::prepareResult()
     Q_EMIT finished();
 }
 
-void ThumbnailResponse::doCancel()
-{
-    if (!Controller::instance().activeConnection()) {
-        return;
-    }
-    // Runs in the main thread, not QML thread
-    if (job) {
-        Q_ASSERT(QThread::currentThread() == job->thread());
-        job->abandon();
-    }
-}
-
 QQuickTextureFactory *ThumbnailResponse::textureFactory() const
 {
     QReadLocker _(&lock);
@@ -122,12 +116,9 @@ QString ThumbnailResponse::errorString() const
     return errorStr;
 }
 
-void ThumbnailResponse::cancel()
-{
-    QMetaObject::invokeMethod(this, &ThumbnailResponse::doCancel, Qt::QueuedConnection);
-}
-
 QQuickImageResponse *MatrixImageProvider::requestImageResponse(const QString &id, const QSize &requestedSize)
 {
     return new ThumbnailResponse(id, requestedSize);
 }
+
+#include "moc_matriximageprovider.cpp"
