@@ -6,7 +6,7 @@
 
 #include <QQuickAsyncImageProvider>
 
-#include <jobs/mediathumbnailjob.h>
+#include <Quotient/jobs/mediathumbnailjob.h>
 
 #include <QReadWriteLock>
 
@@ -15,20 +15,26 @@ namespace Quotient
 class Connection;
 }
 
+/**
+ * @class ThumbnailResponse
+ *
+ * A QQuickImageResponse for an mxc image.
+ *
+ * @sa QQuickImageResponse
+ */
 class ThumbnailResponse : public QQuickImageResponse
 {
     Q_OBJECT
 public:
-    ThumbnailResponse(QString mediaId, QSize requestedSize);
+    explicit ThumbnailResponse(QString mediaId, QSize requestedSize);
     ~ThumbnailResponse() override = default;
 
 private Q_SLOTS:
     void startRequest();
     void prepareResult();
-    void doCancel();
 
 private:
-    const QString mediaId;
+    QString mediaId;
     QSize requestedSize;
     const QString localFile;
     Quotient::MediaThumbnailJob *job = nullptr;
@@ -39,11 +45,22 @@ private:
 
     QQuickTextureFactory *textureFactory() const override;
     QString errorString() const override;
-    void cancel() override;
 };
 
+/**
+ * @class MatrixImageProvider
+ *
+ * A QQuickAsyncImageProvider for mxc images.
+ *
+ * @sa QQuickAsyncImageProvider
+ */
 class MatrixImageProvider : public QQuickAsyncImageProvider
 {
 public:
+    /**
+     * @brief Return a job to provide the image with the given ID.
+     *
+     * @sa QQuickAsyncImageProvider::requestImageResponse
+     */
     QQuickImageResponse *requestImageResponse(const QString &id, const QSize &requestedSize) override;
 };
