@@ -5,10 +5,9 @@
 
 #include "controller.h"
 #include "neochatroom.h"
-#include "neochatuser.h"
 #include "roommanager.h"
-#include <events/roommemberevent.h>
-#include <events/roompowerlevelsevent.h>
+#include <Quotient/events/roommemberevent.h>
+#include <Quotient/events/roompowerlevelsevent.h>
 
 #include <KLocalizedString>
 
@@ -19,6 +18,41 @@ QStringList rainbowColors{"#ff2b00", "#ff5500", "#ff8000", "#ffaa00", "#ffd500",
                           "#00ff2b", "#00ff55", "#00ff80", "#00ffaa", "#00ffd5", "#00ffff", "#00d4ff", "#00aaff", "#007fff", "#0055ff", "#002bff", "#0000ff",
                           "#2a00ff", "#5500ff", "#7f00ff", "#aa00ff", "#d400ff", "#ff00ff", "#ff00d4", "#ff00aa", "#ff0080", "#ff0055", "#ff002b", "#ff0000"};
 
+auto leaveRoomLambda = [](const QString &text, NeoChatRoom *room) {
+    if (text.isEmpty()) {
+        Q_EMIT room->showMessage(NeoChatRoom::Info, i18n("Leaving this room."));
+        room->connection()->leaveRoom(room);
+    } else {
+        QRegularExpression roomRegex(QStringLiteral(R"(^[#!][^:]+:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?)"));
+        auto regexMatch = roomRegex.match(text);
+        if (!regexMatch.hasMatch()) {
+            Q_EMIT room->showMessage(NeoChatRoom::Error,
+                                     i18nc("'<text>' does not look like a room id or alias.", "'%1' does not look like a room id or alias.", text));
+            return QString();
+        }
+        auto leaving = room->connection()->room(text);
+        if (!leaving) {
+            leaving = room->connection()->roomByAlias(text);
+        }
+        if (leaving) {
+            Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Leaving room <roomname>.", "Leaving room %1.", text));
+            room->connection()->leaveRoom(leaving);
+        } else {
+            Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Room <roomname> not found", "Room %1 not found.", text));
+        }
+    }
+    return QString();
+};
+
+auto roomNickLambda = [](const QString &text, NeoChatRoom *room) {
+    if (text.isEmpty()) {
+        Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("No new nickname provided, no changes will happen."));
+    } else {
+        room->connection()->user()->rename(text, room);
+    }
+    return QString();
+};
+
 QVector<ActionsModel::Action> actions{
     Action{
         QStringLiteral("shrug"),
@@ -157,7 +191,6 @@ QVector<ActionsModel::Action> actions{
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18nc("'<text>' does not look like a matrix id.", "'%1' does not look like a matrix id.", text));
                 return QString();
             }
-#ifdef QUOTIENT_07
             const RoomMemberEvent *roomMemberEvent = room->currentState().get<RoomMemberEvent>(text);
             if (roomMemberEvent && roomMemberEvent->membership() == Membership::Invite) {
                 Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("<user> is already invited to this room.", "%1 is already invited to this room.", text));
@@ -167,7 +200,6 @@ QVector<ActionsModel::Action> actions{
                 Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("<user> is banned from this room.", "%1 is banned from this room.", text));
                 return QString();
             }
-#endif
             if (room->localUser()->id() == text) {
                 Q_EMIT room->showMessage(NeoChatRoom::Positive, i18n("You are already in this room."));
                 return QString();
@@ -209,7 +241,6 @@ QVector<ActionsModel::Action> actions{
         kli18n("<room alias or id>"),
         kli18n("Joins the given room"),
     },
-#ifdef QUOTIENT_07
     Action{
         QStringLiteral("knock"),
         [](const QString &text, NeoChatRoom *room) {
@@ -242,7 +273,6 @@ QVector<ActionsModel::Action> actions{
         kli18n("<room alias or id> [<reason>]"),
         kli18n("Requests to join the given room"),
     },
-#endif
     Action{
         QStringLiteral("j"),
         [](const QString &text, NeoChatRoom *room) {
@@ -268,31 +298,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("part"),
-        [](const QString &text, NeoChatRoom *room) {
-            if (text.isEmpty()) {
-                Q_EMIT room->showMessage(NeoChatRoom::Info, i18n("Leaving this room."));
-                room->connection()->leaveRoom(room);
-            } else {
-                QRegularExpression roomRegex(QStringLiteral(R"(^[#!][^:]+:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?)"));
-                auto regexMatch = roomRegex.match(text);
-                if (!regexMatch.hasMatch()) {
-                    Q_EMIT room->showMessage(NeoChatRoom::Error,
-                                             i18nc("'<text>' does not look like a room id or alias.", "'%1' does not look like a room id or alias.", text));
-                    return QString();
-                }
-                auto leaving = room->connection()->room(text);
-                if (!leaving) {
-                    leaving = room->connection()->roomByAlias(text);
-                }
-                if (leaving) {
-                    Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Leaving room <roomname>.", "Leaving room %1.", text));
-                    room->connection()->leaveRoom(leaving);
-                } else {
-                    Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Room <roomname> not found", "Room %1 not found.", text));
-                }
-            }
-            return QString();
-        },
+        leaveRoomLambda,
         false,
         std::nullopt,
         kli18n("[<room alias or id>]"),
@@ -300,31 +306,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("leave"),
-        [](const QString &text, NeoChatRoom *room) {
-            if (text.isEmpty()) {
-                Q_EMIT room->showMessage(NeoChatRoom::Info, i18n("Leaving this room."));
-                room->connection()->leaveRoom(room);
-            } else {
-                QRegularExpression roomRegex(QStringLiteral(R"(^[#!][^:]+:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?)"));
-                auto regexMatch = roomRegex.match(text);
-                if (!regexMatch.hasMatch()) {
-                    Q_EMIT room->showMessage(NeoChatRoom::Error,
-                                             i18nc("'<text>' does not look like a room id or alias.", "'%1' does not look like a room id or alias.", text));
-                    return QString();
-                }
-                auto leaving = room->connection()->room(text);
-                if (!leaving) {
-                    leaving = room->connection()->roomByAlias(text);
-                }
-                if (leaving) {
-                    Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Leaving room <roomname>.", "Leaving room %1.", text));
-                    room->connection()->leaveRoom(leaving);
-                } else {
-                    Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Room <roomname> not found", "Room %1 not found.", text));
-                }
-            }
-            return QString();
-        },
+        leaveRoomLambda,
         false,
         std::nullopt,
         kli18n("[<room alias or id>]"),
@@ -347,14 +329,15 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("roomnick"),
-        [](const QString &text, NeoChatRoom *room) {
-            if (text.isEmpty()) {
-                Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("No new nickname provided, no changes will happen."));
-            } else {
-                room->connection()->user()->rename(text, room);
-            }
-            return QString();
-        },
+        roomNickLambda,
+        false,
+        std::nullopt,
+        kli18n("<display name>"),
+        kli18n("Changes your display name in this room"),
+    },
+    Action{
+        QStringLiteral("myroomnick"),
+        roomNickLambda,
         false,
         std::nullopt,
         kli18n("<display name>"),
@@ -448,14 +431,15 @@ QVector<ActionsModel::Action> actions{
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18nc("'<text>' does not look like a matrix id.", "'%1' does not look like a matrix id.", text));
                 return QString();
             }
-#ifdef QUOTIENT_07
             auto state = room->currentState().get<RoomMemberEvent>(parts[0]);
             if (state && state->membership() == Membership::Ban) {
                 Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("<user> is already banned from this room.", "%1 is already banned from this room.", text));
                 return QString();
             }
-#endif
-            auto plEvent = room->getCurrentState<RoomPowerLevelsEvent>();
+            auto plEvent = room->currentState().get<RoomPowerLevelsEvent>();
+            if (!plEvent) {
+                return QString();
+            }
             if (plEvent->ban() > plEvent->powerLevelForUser(room->localUser()->id())) {
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("You are not allowed to ban users from this room."));
                 return QString();
@@ -485,18 +469,19 @@ QVector<ActionsModel::Action> actions{
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18nc("'<text>' does not look like a matrix id.", "'%1' does not look like a matrix id.", text));
                 return QString();
             }
-            auto plEvent = room->getCurrentState<RoomPowerLevelsEvent>();
+            auto plEvent = room->currentState().get<RoomPowerLevelsEvent>();
+            if (!plEvent) {
+                return QString();
+            }
             if (plEvent->ban() > plEvent->powerLevelForUser(room->localUser()->id())) {
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("You are not allowed to unban users from this room."));
                 return QString();
             }
-#ifdef QUOTIENT_07
             auto state = room->currentState().get<RoomMemberEvent>(text);
             if (state && state->membership() != Membership::Ban) {
                 Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("<user> is not banned from this room.", "%1 is not banned from this room.", text));
                 return QString();
             }
-#endif
             room->unban(text);
             Q_EMIT room->showMessage(NeoChatRoom::Positive, i18nc("<username> was unbanned from this room.", "%1 was unbanned from this room.", text));
 
@@ -523,13 +508,14 @@ QVector<ActionsModel::Action> actions{
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("You cannot kick yourself from the room."));
                 return QString();
             }
-#ifdef QUOTIENT_07
             if (!room->isMember(parts[0])) {
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18nc("<username> is not in this room", "%1 is not in this room.", parts[0]));
                 return QString();
             }
-#endif
-            auto plEvent = room->getCurrentState<RoomPowerLevelsEvent>();
+            auto plEvent = room->currentState().get<RoomPowerLevelsEvent>();
+            if (!plEvent) {
+                return QString();
+            }
             auto kick = plEvent->kick();
             if (plEvent->powerLevelForUser(room->localUser()->id()) < kick) {
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("You are not allowed to kick users from this room."));
