@@ -5,6 +5,9 @@
 
 #include "controller.h"
 
+#include <QDateTime>
+#include <QLocale>
+
 #include <KLocalizedString>
 
 #include <Quotient/csapi/device_management.h>
@@ -52,10 +55,17 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
         } else {
             return false;
         }
+    case TimestampString:
+        if (device.lastSeenTs) {
+            return QDateTime::fromMSecsSinceEpoch(*device.lastSeenTs).toString(QLocale().dateTimeFormat(QLocale::ShortFormat));
+        } else {
+            return false;
+        }
     case Type:
         if (device.deviceId == m_connection->deviceId()) {
             return This;
         }
+#ifdef Quotient_E2EE_ENABLED
         if (!m_connection->isKnownE2eeCapableDevice(m_connection->userId(), device.deviceId)) {
             return Unencrypted;
         }
@@ -64,6 +74,9 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
         } else {
             return Unverified;
         }
+#else
+        return Unverified;
+#endif
     }
     return {};
 }
@@ -81,6 +94,7 @@ QHash<int, QByteArray> DevicesModel::roleNames() const
         {DisplayName, "displayName"},
         {LastIp, "lastIp"},
         {LastTimestamp, "lastTimestamp"},
+        {TimestampString, "timestamp"},
         {Type, "type"},
     };
 }
@@ -147,6 +161,7 @@ void DevicesModel::setConnection(Connection *connection)
     Q_EMIT connectionChanged();
     fetchDevices();
 
+#ifdef Quotient_E2EE_ENABLED
     connect(m_connection, &Connection::sessionVerified, this, [this](const QString &userId, const QString &deviceId) {
         Q_UNUSED(deviceId);
         if (userId == Controller::instance().activeConnection()->userId()) {
@@ -156,6 +171,7 @@ void DevicesModel::setConnection(Connection *connection)
     connect(m_connection, &Connection::finishedQueryingKeys, this, [this]() {
         fetchDevices();
     });
+#endif
 }
 
 #include "moc_devicesmodel.cpp"
