@@ -10,32 +10,24 @@ StateModel::StateModel(QObject *parent)
 
 QHash<int, QByteArray> StateModel::roleNames() const
 {
-    return {{TypeRole, "type"}, {StateKeyRole, "stateKey"}, {SourceRole, "source"}};
+    return {{TypeRole, "type"}, {StateKeyRole, "stateKey"}};
 }
 QVariant StateModel::data(const QModelIndex &index, int role) const
 {
-#ifdef QUOTIENT_07
     auto row = index.row();
     switch (role) {
     case TypeRole:
-        return m_room->currentState().events().keys()[row].first;
+        return m_stateEvents[row].first;
     case StateKeyRole:
-        return m_room->currentState().events().keys()[row].second;
-    case SourceRole:
-        return QJsonDocument(m_room->currentState().events()[m_room->currentState().events().keys()[row]]->fullJson()).toJson();
+        return m_stateEvents[row].second;
     }
-#endif
     return {};
 }
 
 int StateModel::rowCount(const QModelIndex &parent) const
 {
     Q_UNUSED(parent);
-#ifdef QUOTIENT_07
     return m_room->currentState().events().size();
-#else
-    return 0;
-#endif
 }
 
 NeoChatRoom *StateModel::room() const
@@ -48,9 +40,21 @@ void StateModel::setRoom(NeoChatRoom *room)
     m_room = room;
     Q_EMIT roomChanged();
     beginResetModel();
+    m_stateEvents.clear();
+    m_stateEvents = m_room->currentState().events().keys();
     endResetModel();
-    connect(room, &NeoChatRoom::changed, this, [=] {
+    connect(room, &NeoChatRoom::changed, this, [this] {
         beginResetModel();
+        m_stateEvents.clear();
+        m_stateEvents = m_room->currentState().events().keys();
         endResetModel();
     });
 }
+
+QByteArray StateModel::stateEventJson(const QModelIndex &index)
+{
+    auto row = index.row();
+    return QJsonDocument(m_room->currentState().events()[m_stateEvents[row]]->fullJson()).toJson();
+}
+
+#include "moc_statemodel.cpp"
