@@ -5,28 +5,32 @@
 
 #include <QObject>
 
-#include <user.h>
+#include <Quotient/user.h>
 
+/**
+ * @class NeoChatUser
+ *
+ * A class inherited from Quotient::User to add a user color function.
+ *
+ * @sa Quotient::User
+ */
 class NeoChatUser : public Quotient::User
 {
     Q_OBJECT
+
+    /**
+     * @brief The color to use for the user.
+     */
     Q_PROPERTY(QColor color READ color WRITE setColor NOTIFY colorChanged)
-    // Only valid for the local user
-    Q_PROPERTY(QString accountLabel READ accountLabel WRITE setAccountLabel NOTIFY accountLabelChanged)
 public:
-    NeoChatUser(QString userId, Quotient::Connection *connection);
+    explicit NeoChatUser(QString userId, Quotient::Connection *connection);
 
 public Q_SLOTS:
     QColor color();
     void setColor(const QColor &color);
 
-    // Only valid for the local user
-    QString accountLabel() const;
-    void setAccountLabel(const QString &accountLabel);
-
 Q_SIGNALS:
     void colorChanged(QColor _t1);
-    void accountLabelChanged();
 
 private:
     QColor m_color;
