@@ -61,7 +61,7 @@ RemoteMatches Runner::Match(const QString &searchTerm)
         const QString name = m_model.data(m_model.index(i, 0), RoomListModel::DisplayNameRole).toString();
 
         match.iconName = QStringLiteral("org.kde.neochat");
-        match.id = m_model.data(m_model.index(i, 0), RoomListModel::IdRole).toString();
+        match.id = m_model.data(m_model.index(i, 0), RoomListModel::RoomIdRole).toString();
         match.text = name;
         match.relevance = 1;
         const RemoteImage remoteImage = serializeImage(m_model.data(m_model.index(i, 0), RoomListModel::AvatarImageRole).value<QImage>());
@@ -94,3 +94,5 @@ void Runner::Run(const QString &id, const QString &actionId)
 
     WindowController::instance().showAndRaiseWindow(QString());
 }
+
+#include "moc_runner.cpp"
