@@ -4,6 +4,8 @@
 #include "trayicon_sni.h"
 #include <KWindowSystem>
 
+#include "windowcontroller.h"
+
 TrayIcon::TrayIcon(QObject *parent)
     : KStatusNotifierItem(parent)
 {
@@ -13,6 +15,14 @@ TrayIcon::TrayIcon(QObject *parent)
         KWindowSystem::setCurrentXdgActivationToken(providedToken());
         Q_EMIT showWindow();
     });
+
+    connect(&WindowController::instance(), &WindowController::windowChanged, this, [this] {
+#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
+        setAssociatedWindow(WindowController::instance().window());
+#else
+        Q_UNUSED(this);
+#endif
+    });
 }
 
 void TrayIcon::show()
@@ -24,3 +34,5 @@ void TrayIcon::hide()
 {
     setStatus(Passive);
 }
+
+#include "moc_trayicon_sni.cpp"
